#ifndef __CONFIG_H__
#define __CONFIG_H__

/* configured by cmake, do not edit */

//#define ALBUM_VERSION ""
//#define VERSION        ""
/* #undef USE_DXCB */
/* #undef DMR_DEBUG */

/* only defined when build with flatpak */
/* #undef DTK_DMAN_PORTAL */

/*#define RADIUS 0*/
/*#define RADIUS_MV 18*/

#define USE_DTK
/* #undef NO_DTK */


#define TRANSFILEPATH "/usr/share/deepin-draw/translations"
#define TOOLPLUGINPATH "/usr/lib///plugins/tools"
#define TOOLPLUGINTRANSPATH "/usr/share/deepin-draw/translations/plugins/tools"
#endif  /* __CONFIG_H__ */
