// SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#pragma once
#include "config.h"
#ifndef USE_DTK
#define DLineEdit QLineEdit
#define DDialog QDialog
#define DSlider QSlider
#define DLabel QLabel
#define DFrame QFrame
#endif
