// SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#ifndef PAGESCENE_P_H
#define PAGESCENE_P_H
#include <pagescene.h>

//class PageScene::PageScene_private
//{
//public:
//    explicit PageScene_private(PageScene*qq);

//    PageScene* q;
//};

#endif // PAGESCENE_P_H
